/**
 * Reporting plugin
 *
 * Use PhantomJS to produce reports about checks and deliver them to some channels.
 *
 * Installation
 * ------------
 *
 * Usage
 * -----
 */
var fs   = require('fs');
var ejs  = require('ejs');
var yaml = require('js-yaml');
var express = require('express');

var Check = require('../../models/check');
var Ping = require('../../models/ping');

exports.initWebApp = function(options) {

  var config = options.config;
  var count = 0;

  Ping.on('afterInsert', function(ping) {
    Check.findById(ping.check, function(err, check) {
      if (err) {
        return;
      }
      if (!check) {
        return;
      }

      var output = '/var/log/nodejs/reports/' + check.name + '.png',
          date = Date.now();
      if (fs.existsSync(output)) {
        var stat = fs.statSync(output),
            fdate = new Date(stat.mtime);
        if ((fdate + 21600000) > date) {
          // Only refresh the file if it's old than 6 hours
          return;
        }
      }

      if (count > 0) {
        return;
      }
      count++;
      console.log('PhantomJS started, currently ' + count + ' processes.');

      // Create the report
      var phantomjs = require('phantomjs-prebuilt'),
          path = require('path'),
          program = phantomjs.exec(path.join(__dirname, 'phantomjs-script.js'), config['url'], config['basicAuth']['username'], config['basicAuth']['password'], check._id, output, 'month', date);

      program.on('exit', function() {
        count--;
        console.log('PhantomJS finished');
      });
    });
  });

};
