/**
 * Tracker plugin
 *
 * Poll issues, notifications, feed items from external sources
 *
 * Installation
 * ------------
 *
 * Usage
 * -----
 */
var fs   = require('fs');
var ejs  = require('ejs');
var yaml = require('js-yaml');
var express = require('express');

var Check = require('../../models/check');
var Ping = require('../../models/ping');

var SourceCollection = require('./sources/sourcesCollection');
var template = fs.readFileSync(__dirname + '/views/_detailsEdit.ejs', 'utf8');

var sourceCollection = new SourceCollection();

exports.initWebApp = function(options) {

  var api = options.api;
  var dashboard = options.dashboard;

  api.on('populateFromDirtyCheck', function(checkDocument, dirtyCheck, type) {
    dashboard.emit('populateFromDirtyCheck', checkDocument, dirtyCheck, type);
  });

  dashboard.on('populateFromDirtyCheck', function(checkDocument, dirtyCheck, type) {
    if (type !== 'http' && type !== 'https') return;
    if (!dirtyCheck.tracker_source || dirtyCheck.tracker_source == '') {
      checkDocument.setPollerParam('tracker_source', '');
      return;
    }
    checkDocument.setPollerParam('tracker_source', dirtyCheck.tracker_source);
    checkDocument.setPollerParam('tracker_timeshift', dirtyCheck.tracker_timeshift);
    if (dirtyCheck.tracker_lastActivity) {
      checkDocument.setPollerParam('tracker_lastActivity', dirtyCheck.tracker_lastActivity);
    }
    checkDocument.setPollerParam('tracker_language', dirtyCheck.tracker_language);
    checkDocument.setPollerParam('tracker_hidemine', dirtyCheck.tracker_hidemine);
    checkDocument.setPollerParam('tracker_username', dirtyCheck.tracker_username);
    checkDocument.setPollerParam('tracker_password', dirtyCheck.tracker_password);
    checkDocument.setPollerParam('tracker_project_include', dirtyCheck.tracker_project_include);
    checkDocument.setPollerParam('tracker_project_exclude', dirtyCheck.tracker_project_exclude);
  });

  dashboard.on('checkEdit', function(type, check, partial) {
    if (type !== 'http' && type !== 'https') return;
    partial.push(ejs.render(template, { locals: { check: check, sourceCollection: sourceCollection } }));
  });

  Ping.on('afterInsert', function(ping) {
    Check.findById(ping.check, function(err, check) {
      if (err) {
        return;
      }
      if (!check) {
        return;
      }
      if (check.type !== 'http' && check.type !== 'https') return;
      var source = check.pollerParams && check.pollerParams.tracker_source;
      if (!source) return;
      if (source == '') return;

      // Now poll the tracker source
      var lastActivity = check.pollerParams.tracker_lastActivity || 0;
      var Source = sourceCollection.getForType(source);
      var s = new Source(dashboard, check, lastActivity);

      // This is running asynchronously
      s.poll();
    });
  });

};
