/**
 * Module dependencies.
 */
var Check = require('../../../models/check');

/**
 * Base TrackerSource constructor
 *
 * @param dashboard
 * @param {Check} check
 * @param {Integer} lastActivity
 * @api   public
 */
function BaseTrackerSource(dashboard, check, lastActivity) {
  this.debugEnabled = false;

  this.dashboard = dashboard;
  this.check = check;
  this.lastActivity = lastActivity;
  this.url = check.url;
  this.timeshift = check.pollerParams.tracker_timeshift || 0;
  this.language = check.pollerParams.tracker_language || 'en';
  this.hideMine = ((check.pollerParams.tracker_hidemine || 0) == 1);
  this.username = check.pollerParams.tracker_username;
  this.password = check.pollerParams.tracker_password;
  this.project_include = check.pollerParams.tracker_project_include;
  this.project_exclude = check.pollerParams.tracker_project_exclude;
  this.unresolvedOnly = false;

  this.initialize();
}

BaseTrackerSource.strings = {};

/**
 * Initializer method
 *
 * Override this method in child classes to prepare the target property.
 * @api   public
 */
BaseTrackerSource.prototype.initialize = function() {};

BaseTrackerSource.prototype.setLastActivity = function(lastActivity) {
  this.lastActivity = lastActivity;
  this.check.setPollerParam('tracker_lastActivity', lastActivity);
  this.check.save();
};

BaseTrackerSource.prototype.poll = function() {};

BaseTrackerSource.prototype.readItems = function(items, sort) {
  if (sort) {
    var source = this;
    items.sort(function(a, b) {return source.compareDate(a, b);});
  }
  this.debug('Last activity raw: ' + this.lastActivity);
  this.debug('Last activity: ' + this.formatDate(this.lastActivity));

  var i, item;
  for (i in items) {
    item = items[i];
    if (this.isItemNew(item)) {
      if (this.notifyItem(item)) {
        this.debug('Notify');
        this.dashboard.emit('newTrackerItem', this, item);
      }

      // If notification was successful, remember the last activity time
      this.setLastActivity(this.getItemDate(item));
    }
  }
};

BaseTrackerSource.prototype.notifyItem = function(item) {
  return 'default should be overwritten';
};

BaseTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  return 'default should be overwritten';
};

BaseTrackerSource.prototype.getItemDate = function(item) {
  return Date.now();
};

BaseTrackerSource.prototype.compareDate = function(a, b) {
  return (this.getItemDate(a) - this.getItemDate(b));
};

BaseTrackerSource.prototype.isItemNew = function(item) {
  return (this.getItemDate(item) > this.lastActivity);
};

BaseTrackerSource.prototype.t = function(string) {
  return this.constructor.strings[string][this.language];
};

BaseTrackerSource.prototype.formatDate = function(date) {
  return new Date(date);
};

BaseTrackerSource.prototype.debug = function(message) {
  if (!this.debugEnabled) return;
  console.log(this.constructor.type + ' - ' + this.check._id + ': ' + message);
};

module.exports = BaseTrackerSource;
