/**
 * Module dependencies.
 */
var JiraClient = require('jira-connector');
var util = require('util');
var BaseTrackerSource = require('../baseSource');

/**
 * Jira TrackerSource, to receive latest changes from and report to targets
 *
 * @param {String} id
 * @param {Integer} lastActivity
 * @param {String} url
 * @param {Array} params
 * @api   public
 */
function JiraTrackerSource(id, lastActivity, url, params) {
  JiraTrackerSource.super_.call(this, id, lastActivity, url, params);
}

util.inherits(JiraTrackerSource, BaseTrackerSource);

JiraTrackerSource.type = 'Jira';

JiraTrackerSource.prototype.poll = function() {
  var source = this;
  var jira = new JiraClient( {
    host: source.url.substr(source.url.search('://') + 3),
    basic_auth: {
      username: source.username,
      password: source.password
    }
  });
  var jql = 'assignee = ' + source.username;
  if (source.unresolvedOnly) {
    jql += ' AND resolution = unresolved';
  }
  if (source.lastActivity != 0) {
    jql += ' AND updated > "' + source.lastActivity.substr(0, 10) + ' ' + source.lastActivity.substr(11, 5) + '"';
  }
  jira.search.search({jql: jql}, function(error, response) {
    if (error) {
      source.debug(error);
      return;
    }
    if (response === undefined || response.issues === undefined) {
      return;
    }
    source.readItems(response.issues, true);
  });
};

JiraTrackerSource.prototype.notifyItem = function(item) {
  return true;
};

JiraTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  return linkCB(item.key, this.url + '/browse/' + item.key) + ' ' + item.fields.summary;
};

JiraTrackerSource.prototype.getItemDate = function(item) {
  return item.fields.updated;
};

JiraTrackerSource.prototype.compareDate = function(a, b) {
  return (Date.parse(this.getItemDate(a)) - Date.parse(this.getItemDate(b)));
};

JiraTrackerSource.prototype.isItemNew = function(item) {
  return (Date.parse(this.getItemDate(item)) > Date.parse(this.lastActivity));
};

JiraTrackerSource.prototype.formatDate = function(timestamp) {
  return timestamp;
};

module.exports = JiraTrackerSource;
