/**
 * Module dependencies.
 */
var request = require('request');
var util = require('util');
var BaseTrackerSource = require('../baseSource');

/**
 * Redmine TrackerSource, to receive latest changes from and report to targets
 *
 * @param {String} id
 * @param {Integer} lastActivity
 * @param {String} url
 * @param {Array} params
 * @api   public
 */
function RedmineTrackerSource(id, lastActivity, url, params) {
  RedmineTrackerSource.super_.call(this, id, lastActivity, url, params);
}

util.inherits(RedmineTrackerSource, BaseTrackerSource);

RedmineTrackerSource.type = 'Redmine';

RedmineTrackerSource.prototype.poll = function() {
  var source = this;
  var url = source.url + '/issues.json?key=' + source.password + '&sort=updated_on&updated_on=%3E%3D' + source.formatDate(source.lastActivity);
  if (source.project_include) {
    url += "&project_id=" + source.project_include;
  }
  source.debug(url);
  request
    .get(url,
      function(error, response, body) {
        var res = JSON.parse(body);
        source.debug(body);
        source.readItems(res.issues, false);
      })
    .on('error', function(e) {
      source.debug('ERROR: ' + e.message);
    });
};

RedmineTrackerSource.prototype.notifyItem = function(item) {
  return !(this.hideMine && (item['assigned_to']['name'] == this.username));
};

RedmineTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  return item['assigned_to']['name'] + ': ' + linkCB(item['id'], this.url + '/issues/' + item['id']) + ' ' + item['subject'];
};

RedmineTrackerSource.prototype.getItemDate = function(item) {
  return item['updated_on'];
};

RedmineTrackerSource.prototype.formatDate = function(timestamp) {
  return timestamp || new Date().toISOString().substr(0, 19) + 'Z';
};

module.exports = RedmineTrackerSource;
