/**
 * Module dependencies.
 */
var FeedParser = require('feedparser');
var request = require('request');
var util = require('util');
var BaseTrackerSource = require('../baseSource');

/**
 * RSS TrackerSource, to receive latest changes from and report to targets
 *
 * @param {String} id
 * @param {Integer} lastActivity
 * @param {String} url
 * @param {Array} params
 * @api   public
 */
function RssTrackerSource(id, lastActivity, url, params) {
  RssTrackerSource.super_.call(this, id, lastActivity, url, params);
}

util.inherits(RssTrackerSource, BaseTrackerSource);

RssTrackerSource.type = 'RSS';
RssTrackerSource.blacklist = [
  'Keine Einträge'
];

RssTrackerSource.prototype.poll = function() {
  var source = this;
  var items = [];
  var feedparser = new FeedParser();
  var req = request(this.url)
    .on('response', function (res) {
      if (res.statusCode != 200) return;
      this.pipe(feedparser);
    });

  feedparser
    .on('readable', function() {
      while (item = this.read()) {
        items.push(item);
      }
    })
    .on('end', function() {
      source.readItems(items, true);
    });
};

RssTrackerSource.prototype.notifyItem = function(item) {
  var i;
  for (i in RssTrackerSource.blacklist) {
    if (item.title == RssTrackerSource.blacklist[i]) {
      return false;
    }
  }
  return true;
};

RssTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  return item.title + ' ' + linkCB('Details', item.link);
};

RssTrackerSource.prototype.getItemDate = function(item) {
  return Date.parse(item.date);
};

module.exports = RssTrackerSource;
