/**
 * Module dependencies.
 */
var request = require('request');
var util = require('util');
var BaseTrackerSource = require('../baseSource');

/**
 * TeamWork TrackerSource, to receive latest changes from and report to targets
 *
 * @param {String} id
 * @param {Integer} lastActivity
 * @param {String} url
 * @param {Array} params
 * @api   public
 */
function TeamWorkTrackerSource(id, lastActivity, url, params) {
  TeamWorkTrackerSource.super_.call(this, id, lastActivity, url, params);
}

util.inherits(TeamWorkTrackerSource, BaseTrackerSource);

TeamWorkTrackerSource.type = 'TeamWork';

TeamWorkTrackerSource.prototype.poll = function() {
  var source = this;
  var url = source.url + '/projects/' + source.project_include + '/latestActivity.json';
  source.debug(url);
  request
    .get({
        url: url,
        headers : {
          "Authorization" : 'BASIC ' + new Buffer(source.password + ':xxx').toString('base64')
        }
      },
      function(error, response, body) {
        if (error || response.statusCode !== 200) return;

        var res = JSON.parse(body);
        source.debug(body);
        source.readItems(res.activity, true);
      })
    .on('error', function(e) {
      source.debug('ERROR: ' + e.message);
    });
};

TeamWorkTrackerSource.prototype.notifyItem = function(item) {
  return !(this.hideMine && (item['userid'] == this.username));

};

TeamWorkTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  var
    activitytype = item['activitytype'],// new, edited, completed, reopened
    type = item['type'],// task, task_comment, file
    description = item['description'],
    extradescription = item['extradescription'],
    from = item['fromusername'],
    link = this.url + '/' + item['link'],
    avatar = imageCB('', item['from-user-avatar-url']),
    action = '',
    title = '',
    detail = '',
    output = avatar + ' ' + from + ' ';

  if (type == 'file' || activitytype == 'edited') {
    return false;
  }

  if (type == 'task_comment') {
    if (activitytype == 'new') {
      action = 'commented on';
    }
    else {
      action = 'edited comment on';
    }
    title = extradescription;
    detail = description;
  }
  else {
    if (activitytype == 'new') {
      action = 'created';
    }
    else {
      action = 'edited';
    }
    title = description
  }

  if (action == 'edited') {
    return false;
  }

  output += action + ' ' + linkCB(title, link);
  if (detail != '') {
    output += ': ' + detail;
  }
  return output;
};

TeamWorkTrackerSource.prototype.getItemDate = function(item) {
  return Date.parse(item['datetime']);
};

module.exports = TeamWorkTrackerSource;
